@@ -1,15 +1,15 @@
 #!/usr/bin/env python
 #
-# Copyright 2011-2013 Ghent University
+# Copyright 2011-2020 Ghent University
 #
 # This file is part of vsc-base,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
 # with support of Ghent University (http://ugent.be/hpc),
-# the Flemish Supercomputer Centre (VSC) (https://vscentrum.be/nl/en),
-# the Hercules foundation (http://www.herculesstichting.be/in_English)
+# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
+# the Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/vsc-base
+# https://github.com/hpcugent/vsc-base
 #
 # vsc-base is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Library General Public License as
@@ -36,8 +36,9 @@ then use mpi to get these environment variables to the clients.
 
 @author: Jens Timmerman (Ghent University)
 """
+from __future__ import print_function
 from optparse import OptionParser
-from vsc import fancylogger
+from vsc.utils import fancylogger
 from vsc.utils.daemon import Daemon
 import logging
 import os
@@ -91,9 +92,9 @@ class LogDaemon(Daemon):
 
         # get socket
         self.socket_.bind((self.hostname, self.port))
-        print "FANCYLOG_SERVER_PID=%s" % self.pidfile
-        print "FANCYLOG_SERVER=%s:%d" % (socket.gethostname(), self.socket_.getsockname()[-1])
-        print "FANCYLOG_SERVER_LOGFILE=%s" % self.logfile
+        print("FANCYLOG_SERVER_PID=%s" % self.pidfile)
+        print("FANCYLOG_SERVER=%s:%d" % (socket.gethostname(), self.socket_.getsockname()[-1]))
+        print("FANCYLOG_SERVER_LOGFILE=%s" % self.logfile)
         sys.stdout.flush()
 
         # Start the daemon
@@ -114,7 +115,7 @@ class LogDaemon(Daemon):
                 self.logger.handle(logrecord)
             except (KeyboardInterrupt, SystemExit):
                 raise
-            except:
+            except Exception:
                 traceback.print_exc()
 
 
