@@ -1,4 +1,5 @@
-# #
+### External compatible license
+#
 # Copyright 2005 Josiah Carlson
 # The Asynchronous Python Subprocess recipe was originally created by Josiah Carlson.
 # and released under the GNU Library General Public License v2 or any later version
@@ -6,13 +7,13 @@
 #
 # http://code.activestate.com/recipes/440554/
 #
-# Copyright 2009-2013 Ghent University
+# Copyright 2009-2020 Ghent University
 #
 # This file is part of vsc-base,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
 # with support of Ghent University (http://ugent.be/hpc),
 # the Flemish Supercomputer Centre (VSC) (https://vscentrum.be/nl/en),
-# the Hercules foundation (http://www.herculesstichting.be/in_English)
+# the Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
 # http://github.com/hpcugent/vsc-base
@@ -29,7 +30,7 @@
 #
 # You should have received a copy of the GNU Library General Public License
 # along with vsc-base. If not, see <http://www.gnu.org/licenses/>.
-# #
+#
 
 """
 Module to allow Asynchronous subprocess use on Windows and Posix platforms
@@ -108,8 +109,9 @@ class Popen(subprocess.Popen):
 
         try:
             written = os.write(self.stdin.fileno(), inp)
-        except OSError, why:
-            if why[0] == errno.EPIPE:  # broken pipe
+            self.stdin.flush()
+        except OSError as why:
+            if why.args[0] == errno.EPIPE:  # broken pipe
                 return self._close('stdin')
             raise
 
@@ -184,8 +186,11 @@ def send_all(p, data):
     """
     Send data to process p
     """
+    allsent = 0
     while len(data):
         sent = p.send(data)
         if sent is None:
             raise Exception(MESSAGE)
+        allsent += sent
         data = buffer(data, sent)
+    return allsent
