@@ -1,15 +1,14 @@
-#!/usr/bin/env python
-# #
-# Copyright 2012-2013 Ghent University
+#
+# Copyright 2012-2020 Ghent University
 #
 # This file is part of vsc-base,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
 # with support of Ghent University (http://ugent.be/hpc),
-# the Flemish Supercomputer Centre (VSC) (https://vscentrum.be/nl/en),
-# the Hercules foundation (http://www.herculesstichting.be/in_English)
+# the Flemish Supercomputer Centre (VSC) (https://www.vscentrum.be),
+# the Flemish Research Foundation (FWO) (http://www.fwo.be/en)
 # and the Department of Economy, Science and Innovation (EWI) (http://www.ewi-vlaanderen.be/en).
 #
-# http://github.com/hpcugent/vsc-base
+# https://github.com/hpcugent/vsc-base
 #
 # vsc-base is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Library General Public License as
@@ -23,7 +22,7 @@
 #
 # You should have received a copy of the GNU Library General Public License
 # along with vsc-base. If not, see <http://www.gnu.org/licenses/>.
-# #
+#
 """
 Various functions that are missing from the default Python library.
 
@@ -41,24 +40,45 @@ Various functions that are missing from the default Python library.
 @author: Stijn De Weirdt (Ghent University)
 """
 import shlex
-import subprocess
 import time
+try:
+    from shlex import quote  # python 3.3
+except ImportError:
+    from pipes import quote  # python 2.7
+
+from collections import namedtuple, Mapping
 
 from vsc.utils import fancylogger
 from vsc.utils.frozendict import FrozenDict
 
 
-def any(ls):
-    """Reimplementation of 'any' function, which is not available in Python 2.4 yet."""
-
-    return sum([bool(x) for x in ls]) != 0
+_log = fancylogger.getLogger('vsc.utils.missing')
 
 
-def all(ls):
-    """Reimplementation of 'all' function, which is not available in Python 2.4 yet."""
+def is_string(item):
+    """Check whether specified value is a string or not."""
+    return isinstance(item, basestring)
 
-    return sum([bool(x) for x in ls]) == len(ls)
 
+def partial(func, *args, **keywords):
+    """
+    Return a new partial object which when called will behave like func called with the positional arguments args
+    and keyword arguments keywords. If more arguments are supplied to the call, they are appended to args. If additional
+    keyword arguments are supplied, they extend and override keywords.
+    new in python 2.5, from https://docs.python.org/2/library/functools.html#functools.partial
+    """
+    def newfunc(*fargs, **fkeywords):
+        newkeywords = keywords.copy()
+        newkeywords.update(fkeywords)
+        return func(*(args + fargs), **newkeywords)
+    newfunc.func = func
+    newfunc.args = args
+    newfunc.keywords = keywords
+    return newfunc
+
+# Placeholder, used to have implementations for any and all that were missing in py24
+any = any # pylint: disable=redefined-builtin
+all = all # pylint: disable=redefined-builtin
 
 def nub(list_):
     """Returns the unique items of a list of hashables, while preserving order of
@@ -122,7 +142,8 @@ class Monoid(object):
     def __init__(self, null, mappend):
         """Initialise.
 
-        @type null: default element of some data type, e.g., [] for list or 0 for int (identity element in an Abelian group)
+        @type null: default element of some data type, e.g., [] for list or 0 for int
+            (identity element in an Abelian group)
         @type op: mappend operation to combine two elements of the target datatype
         """
         self.null = null
@@ -234,7 +255,7 @@ class FrozenDictKnownKeys(FrozenDict):
 
         # handle unknown keys: either ignore them or raise an exception
         tmpdict = dict(*args, **kwargs)
-        unknown_keys = [key for key in tmpdict.keys() if not key in self.KNOWN_KEYS]
+        unknown_keys = [key for key in tmpdict.keys() if key not in self.KNOWN_KEYS]
         if unknown_keys:
             if ignore_unknown_keys:
                 for key in unknown_keys:
@@ -246,12 +267,12 @@ class FrozenDictKnownKeys(FrozenDict):
                 self.log.raiseException(msg, exception=KeyError)
 
         super(FrozenDictKnownKeys, self).__init__(tmpdict)
-
+    # pylint: disable=arguments-differ
     def __getitem__(self, key, *args, **kwargs):
         """Redefine __getitem__ to provide a better KeyError message."""
         try:
             return super(FrozenDictKnownKeys, self).__getitem__(key, *args, **kwargs)
-        except KeyError, err:
+        except KeyError as err:
             if key in self.KNOWN_KEYS:
                 raise KeyError(err)
             else:
@@ -260,27 +281,53 @@ class FrozenDictKnownKeys(FrozenDict):
 
 
 def shell_quote(x):
-    """Add quotes so it can be apssed to shell"""
-    # use undocumented subprocess API call to quote whitespace (executed with Popen(shell=True))
-    # (see http://stackoverflow.com/questions/4748344/whats-the-reverse-of-shlex-split for alternatives if needed)
-    return subprocess.list2cmdline([str(x)])
+    """Add quotes so it can be passed to shell"""
+    return quote(str(x))
 
 
 def shell_unquote(x):
     """Take a literal string, remove the quotes as if it were passed by shell"""
     # it expects a string
-    return shlex.split(str(x))[0]
+    return ' '.join(shlex.split(str(x)))
 
 
-def get_subclasses(klass):
-    """Get all subclasses recursively"""
-    res = []
-    for cl in klass.__subclasses__():
-        res.extend(get_subclasses(cl))
-        res.append(cl)
+def get_class_for(modulepath, class_name):
+    """
+    Get class for a given Python class name and Python module path.
+
+    @param modulepath: Python module path (e.g., 'vsc.utils.generaloption')
+    @param class_name: Python class name (e.g., 'GeneralOption')
+    """
+    # try to import specified module path, reraise ImportError if it occurs
+    try:
+        module = __import__(modulepath, globals(), locals(), [''])
+    except ImportError as err:
+        raise ImportError(err)
+    # try to import specified class name from specified module path, throw ImportError if this fails
+    try:
+        klass = getattr(module, class_name)
+    except AttributeError as err:
+        raise ImportError("Failed to import %s from %s: %s" % (class_name, modulepath, err))
+    return klass
+
+
+def get_subclasses_dict(klass, include_base_class=False):
+    """Get dict with subclasses per classes, recursively from the specified base class."""
+    res = {}
+    subclasses = klass.__subclasses__()
+    if include_base_class:
+        res.update({klass: subclasses})
+    for subclass in subclasses:
+        # always include base class for recursive call
+        res.update(get_subclasses_dict(subclass, include_base_class=True))
     return res
 
 
+def get_subclasses(klass, include_base_class=False):
+    """Get list of all subclasses, recursively from the specified base class."""
+    return get_subclasses_dict(klass, include_base_class=include_base_class).keys()
+
+
 class TryOrFail(object):
     """
     Perform the function n times, catching each exception in the exception tuple except on the last try
@@ -292,17 +339,16 @@ class TryOrFail(object):
         self.sleep = sleep
 
     def __call__(self, function):
-        def new_function(*args, **kwargs ):
-            log = fancylogger.getLogger(function.__name__)
-            for i in xrange(0,self.n):
+        def new_function(*args, **kwargs):
+            for i in xrange(0, self.n):
                 try:
                     return function(*args, **kwargs)
-                except self.exceptions, err:
+                except self.exceptions as err:
                     if i == self.n - 1:
                         raise
-                    log.exception("try_or_fail caught an exception - attempt %d: %s" % (i, err))
+                    _log.exception("try_or_fail caught an exception - attempt %d: %s" % (i, err))
                     if self.sleep > 0:
-                        log.warning("try_or_fail is sleeping for %d seconds before the next attempt" % (self.sleep,))
+                        _log.warning("try_or_fail is sleeping for %d seconds before the next attempt" % (self.sleep,))
                         time.sleep(self.sleep)
 
         return new_function
@@ -327,7 +373,18 @@ def topological_sort(graph):
     visited = set()
     for root in graph:
         for node in post_order(graph, root):
-            if not node in visited:
+            if node not in visited:
                 yield node
                 visited.add(node)
 
+
+# https://stackoverflow.com/questions/11351032/namedtuple-and-default-values-for-optional-keyword-arguments
+def namedtuple_with_defaults(typename, field_names, default_values=()):
+    T = namedtuple(typename, field_names)
+    T.__new__.__defaults__ = (None,) * len(T._fields)
+    if isinstance(default_values, Mapping):
+        prototype = T(**default_values)
+    else:
+        prototype = T(*default_values)
+    T.__new__.__defaults__ = tuple(prototype)
+    return T
